#ifndef MODPERL_HOOKS_H
#define MODPERL_HOOKS_H

/*
 * *********** WARNING **************
 * This file generated by ModPerl::Code/0.01
 * Any changes made here will be lost
 * ***********************************
 * 01: lib/ModPerl/Code.pm:740
 * 02: lib/ModPerl/Code.pm:766
 * 03: Makefile.PL:391
 * 04: Makefile.PL:102
 */


#define MP_HANDLER_NUM_CONNECTION 1

#define MP_HANDLER_TYPE_CONNECTION 1

#define MP_PROCESS_CONNECTION_HANDLER 0

#define MP_HANDLER_NUM_FILES 2

#define MP_HANDLER_TYPE_FILES 2

#define MP_OPEN_LOGS_HANDLER 0
#define MP_POST_CONFIG_HANDLER 1

#define MP_HANDLER_NUM_PER_DIR 11

#define MP_HANDLER_TYPE_PER_DIR 3

#define MP_HEADER_PARSER_HANDLER 0
#define MP_ACCESS_HANDLER 1
#define MP_AUTHEN_HANDLER 2
#define MP_AUTHZ_HANDLER 3
#define MP_TYPE_HANDLER 4
#define MP_FIXUP_HANDLER 5
#define MP_RESPONSE_HANDLER 6
#define MP_LOG_HANDLER 7
#define MP_CLEANUP_HANDLER 8
#define MP_INPUT_FILTER_HANDLER 9
#define MP_OUTPUT_FILTER_HANDLER 10

#define MP_HANDLER_NUM_PER_SRV 3

#define MP_HANDLER_TYPE_PER_SRV 4

#define MP_POST_READ_REQUEST_HANDLER 0
#define MP_TRANS_HANDLER 1
#define MP_MAP_TO_STORAGE_HANDLER 2

#define MP_HANDLER_NUM_PRE_CONNECTION 1

#define MP_HANDLER_TYPE_PRE_CONNECTION 5

#define MP_PRE_CONNECTION_HANDLER 0

#define MP_HANDLER_NUM_PROCESS 2

#define MP_HANDLER_TYPE_PROCESS 6

#define MP_CHILD_INIT_HANDLER 0
#define MP_CHILD_EXIT_HANDLER 1

int modperl_process_connection_handler(conn_rec *c);

int modperl_open_logs_handler(apr_pool_t *pconf, apr_pool_t *plog, apr_pool_t *ptemp, server_rec *s);

int modperl_post_config_handler(apr_pool_t *pconf, apr_pool_t *plog, apr_pool_t *ptemp, server_rec *s);

int modperl_header_parser_handler(request_rec *r);

int modperl_access_handler(request_rec *r);

int modperl_authen_handler(request_rec *r);

int modperl_authz_handler(request_rec *r);

int modperl_type_handler(request_rec *r);

int modperl_fixup_handler(request_rec *r);

int modperl_log_handler(request_rec *r);

int modperl_post_read_request_handler(request_rec *r);

int modperl_trans_handler(request_rec *r);

int modperl_map_to_storage_handler(request_rec *r);

int modperl_pre_connection_handler(conn_rec *c, void *csd);

void modperl_child_init_handler(apr_pool_t *p, server_rec *s);
void modperl_register_handler_hooks(void);
const char *modperl_handler_desc_connection(int idx);
const char *modperl_handler_desc_files(int idx);
const char *modperl_handler_desc_per_dir(int idx);
const char *modperl_handler_desc_per_srv(int idx);
const char *modperl_handler_desc_pre_connection(int idx);
const char *modperl_handler_desc_process(int idx);
int modperl_handler_lookup(const char *name, int *type);

#endif /* MODPERL_HOOKS_H */
