#ifndef MODPERL_LARGEFILES_H
#define MODPERL_LARGEFILES_H

/*
 * *********** WARNING **************
 * This file generated by ModPerl::Code/0.01
 * Any changes made here will be lost
 * ***********************************
 * 01: lib/ModPerl/Code.pm:740
 * 02: lib/ModPerl/Code.pm:766
 * 03: Makefile.PL:391
 * 04: Makefile.PL:102
 */


#endif /* MODPERL_LARGEFILES_H */
